/**
 * @content:FaultFilter组件实现文件
 * @time:2024-10-09
 * @author:lxd
 * @version: V1.0
 * @describe:
 * 1#2024-10-09#V1.0#首次生成
 */

//#include "../../externservice/ProtocolI.h"

#include <cmath>
#include <cstdlib>
#include <vector>

#include <fstream>
#include <sstream>
#include <chrono>
#include <iomanip>
#include <cstdlib>
#include "fault_filter.h"

#include "../../common/GUID.cpp"
#include "../../servicemodel/Base.h"
#include "../../servicemodel/Channel.h"
#include "../../servicemodel/Device.h"
#include "../../servicemodel/Item.h"
#include "../../servicemodel/Packet.h"
//#include "../../service/DAControlerI.h"

using namespace std;

static std::string file_save_path = "/home/";

struct fault_data {
public:
    long long time;
    std::string device_name;
    std::string name;
    std::string id;
    std::string value;

    // 默认构造函数
    fault_data() : time(0), device_name(""),     name(""), id(""), value("") {}

    // 带参数的构造函数
    fault_data(long long T, std::string d, std::string n, std::string a, std::string s)
        : time(T),  device_name(d), name(n), id(a), value(s) {}
};

//获取源文件中最后10分钟的数据
std::vector<fault_data> get_data_for_CSV(const std::string& back_filename, long long current_time, int save_time_before)
{
    std::string line;
    long long base_time = current_time - save_time_before*60;
    std::vector<fault_data> records;
    std::ifstream file(back_filename);

    // 读取文件并解析数据
    while (std::getline(file, line))
    {
        std::istringstream ss(line);
        long long time;
        std::string device_name;
        std::string name;
        std::string id;
        std::string value;

        //CSV文件格式为：时间戳,设备名称,采集点名称,ID,值
        if (std::getline(ss, line, ','))
        {
            time = std::stoll(line);
        }

        //最后10分钟的数据进行保存
        if (time < base_time) {
            continue;
        }
        std::getline(ss, device_name, ',');
        std::getline(ss, name, ',');
        std::getline(ss, id, ',');
        std::getline(ss, value, ',');
        if (std::getline(ss, name, ','))
        {
            std::getline(ss, line, ',');
            id = std::stoi(line);
        }
        if (std::getline(ss, line))
        {
            value = std::stod(line);
        }
        records.emplace_back(time, device_name, name, id, value);
    }
    return records;
}
void save_to_CSV(const std::string& filename, const std::vector<fault_data>& dataList)
{
    std::ofstream file(filename,  std::ios::app);

    // 检查文件是否成功打开
    if (!file.is_open()) {
        //无法打开,则创建文件
        FILE *total_fp;
        total_fp = fopen("filename", "a+");
        // 写入CSV头部
        printf("写入CSV文件头部\n");
        file << "time,device_name,Name,id,value\n";
        fclose(total_fp);
    }

    // 写入数据
    for (const auto& data : dataList) {
        file << data.time << "," << data.device_name << "," << data.name << "," << data.id << "," << data.value << "\n";
    }
}

/*
外部调用接口
Item *fault_type: 触发故障的源类型
Item *source_data: 源数据
int save_time_before: 保存故障发生前的数据时间
int save_time_after: 保存故障发生后的数据时间
*/
HRESULT ProtocolImpl::fault_filter(data_item* fault_type, data_item* source_data, int save_time_before, int save_time_after)
{
    int source_len = 0, i = 0, fault_type_number = 0, temp_count = 0;
    std::string fault_value, fault_id;
    std::string st_id;
    std::string file_name;
    std::string fault_name;
    std::string fault_file_name;
    std::vector<fault_data> source_before_data;
    std::vector<fault_data> fault_before_data;
    //故障文件参数
    fault_type_number = fault_type->number;
    //源文件参数
    source_len = source_data->number;
    std::vector<fault_data> v_data(source_len);

    printf("fault_type_number is %d, source_len is %d\n", fault_type_number, source_len);
    //获取当前时间;
    auto now = std::chrono::system_clock::now();
    //转换为秒
    long long current_time = std::chrono::duration_cast<std::chrono::seconds>(now.time_since_epoch()).count();
    // 转换为系统时间
    std::time_t now_time_t = std::chrono::system_clock::to_time_t(now);

    // 将时间转换为字符串格式
    std::tm *now_tm = std::localtime(&now_time_t);
    char time_buffer[80];
    std::strftime(time_buffer, sizeof(time_buffer), "%Y-%m-%d %H:%M:%S", now_tm);
    //故障检测
    for (i = 0; i < fault_type_number; ++i)
    {
        fault_value = fault_type->fault_pitem.at(i).value.c_str();
        fault_name = fault_type->fault_pitem.at(i).name.c_str();
        int fault_value_I = strtol(fault_value.c_str(), NULL, HEX);
        if (fault_value_I == 1) {
            printf("%s 出现故障\n", fault_name.c_str());
            fault_count++;
            temp_count++;
        }
    }

    //记录故障数目
    #if 0
    //比较最后一个故障名称
    if (!fault_id.empty() && back_fault_id != fault_id) {
        back_fault_id = fault_id;
    } else {
    }
    #endif
    //解析当前时刻采集到的数据
    for (i = 0; i < source_len; ++i)
    {
        v_data.at(i).time = current_time;
        v_data.at(i).device_name = source_data->source_pitem.at(i).device_name;
        v_data.at(i).name = source_data->source_pitem.at(i).name;
        v_data.at(i).id = source_data->source_pitem.at(i).id;
        v_data.at(i).value = source_data->source_pitem.at(i).value;
    }

    //第一次保存数据, 更新保存文件名
    if (back_time == 0)
    {
        back_time = current_time;
        file_name = file_save_path + "source_" + time_buffer + ".csv";
        printf("file_name is %s\n", file_name.c_str());
        back_file_name = file_name;
    } else {
        //出现故障, 保存故障数据
        if (fault_count > 0) {
            if (!back_fault_count) {
                //新建故障数据文件
                fault_file_name = file_save_path + "fault_" + time_buffer + ".csv";
                back_fault_file_name = fault_file_name;
                fault_before_data = get_data_for_CSV(back_file_name, current_time, save_time_before);
                fault_save_time_begin = current_time;
                save_to_CSV(back_fault_file_name, fault_before_data);
                back_fault_count = 1;
                fault_number_back = temp_count;
            } else {
                //出现新故障,更新故障保存时间
                if (temp_count > fault_number_back) {
                    fault_save_time_begin = current_time;
                }
                long long differ_fault_time = current_time - fault_save_time_begin;
                if (differ_fault_time > save_time_after * 60) {
                    //达到故障发生后最大记录时间,停止记录数据
                    printf("stop_data_save, 达到故障记录最大时间, 停止记录\n");
                    //故障恢复
                    if (temp_count == 0) {
                        fault_save_time_begin = 0;
                        fault_count = 0;
                        back_fault_count = 0;
                    }
                } else {
                    save_to_CSV(back_fault_file_name, v_data);
                }
            }
        }

        //保存时间大于1小时, 新建文件保存数据
        // 计算时间差
        long long differ_second = current_time - back_time;
        if (differ_second > 120) {
            file_name = file_save_path + "source_" + time_buffer + ".csv";
            back_time = current_time;
            source_before_data = get_data_for_CSV(back_file_name, current_time, save_time_before);
            save_to_CSV(file_name, source_before_data);

            std::string rmfile_demand = "rm '" + back_file_name + "'";
            printf("rm rmfile_demand file is %s\n", rmfile_demand.c_str());
            system(rmfile_demand.c_str());
            back_file_name = file_name;
            return S_OK;
        }
    }

    printf("back_file_name is %s \n", back_file_name.c_str());
    //无故障发生时循环迭代保存一小时数据
    save_to_CSV(back_file_name, v_data);

    return S_OK;

}

HRESULT ProtocolImpl::set_fault_list(vector<Device*> data_list)
{
    ProtocolImpl* pP = new ProtocolImpl();
    data_item *source = new data_item();
    data_item *fault = new data_item();
    std::string dev_name;
    if (data_list.size() <= 0) {
        printf("采集设备列表为空\n");
        return S_FALSE;
    }

    while(1) {
        source->source_pitem.clear();
        fault->fault_pitem.clear();
        source->number = 0;
        fault->number = 0;
        printf("data_list.size() is %d\n", data_list.size());
        for (int i = 0; i < data_list.size(); ++i)
        {
            //设备名称
            dev_name = data_list.at(i)->getBase().getName().c_str();
            //printf("i is %d, 设备名称为%s, count is %d\n", i, dev_name.c_str(), source->number);
            //printf("i = %d, data_list.at(i)->getVpacket().size()", i, data_list.at(i)->getVpacket().size());
            for(int j = 0; j < data_list.at(i)->getVpacket().size(); ++j)
            {
                Packet *t_ppacket = data_list.at(i)->getVpacket().at(j);
                if (t_ppacket == NULL)
                {
                    printf("查找的包为空\n");
                    continue;
                }
                int pitem_count = t_ppacket->getItemCount();

                int pitem_i = 0;
                for (pitem_i = 0; pitem_i < pitem_count; ++pitem_i)
                {
                    std::string t_sId, t_sname, t_svalue;
                    int item_enable = 0, item_alarm_enable = 0;
                    Item *forward_pitem = t_ppacket->getVitem().at(pitem_i);
                    if (forward_pitem != NULL)
                    {
                        Base &t_base = forward_pitem->getBase();
                        t_sId = t_base.getObjid().toString().c_str();
                        t_sname = t_base.getName().c_str();
                        t_svalue = forward_pitem->getValue().c_str();
                        if (t_sname.empty() || t_svalue.empty())
                        {
                            continue;
                        }
                        //printf("t_sId is %s, t_sname is %s, t_svalue is %s\n", t_sId.c_str(), t_sname.c_str(), t_svalue.c_str());
                        //printf("i is %d, 设备名称为%s, count is %d\n", i, dev_name.c_str(), source->number);
                        //保存故障滤波源数据
                        source->number += 1;
                        source->add_source_item(dev_name, t_sname, t_sId, t_svalue);

                        //获取故障类型
                        //获取采集点使能, 采集点报警使能状态
                        item_enable = forward_pitem->getEnable();
                        item_alarm_enable = forward_pitem->getAlarmenable();
                        if (item_enable && item_alarm_enable)
                        {
                            fault->number += 1;
                            fault->add_fault_item(dev_name, t_sname, t_sId, t_svalue);
                        }
                    } else {
                        continue;
                    }
                }
            }
        }
        //临时创建故障检测文件
        //fault->number = 1;
        //std::string test_value = "1";
        //fault->add_fault_item("IO", "干接点", "15a8ef1d-07f9-46ad-8d64-dba326d3adac", test_value);
        printf("before fault.capacity() is %d\n", fault->fault_pitem.capacity());
        if (!source) {
            printf("source is NULL\n");
        }
        if (!fault) {
            printf("fault is NULL\n");
        }
        pP->fault_filter(fault, source, 1, 2);
        secondsSleep(5);
    }

    delete pP;
    delete source;
    delete fault;

    return S_OK;
}

